'===  Class Name:   Rss ===================================================
'    Description:   Rss generator assembly
'   Date Created:   2004.01.01
'	Code Copyright  2004 by Stan Schultes, All Rights Reserved.
'==========================================================================

'This component creates an RSS feed from input data. The calling application must populate the RSS
' header and items list and save the file each time the output is updated. The caller is responsible 
' for ensuring the output directory exists. If the directory doesn't exist or an output error occurs, 
' no file is written.

Imports System.Text     'for Stringbuilder
Imports System.IO       'for Directory,Path,Stream

'manages an RSS output file
Public Class Rss

#Region " Class Declarations "

    'class private variables
    Private mhtRSSItems As New Hashtable()
    Private msRSSHeader As String = String.Empty
    Private msRSSChannel As String = String.Empty
    Private msRSSFooter As String = String.Empty
    'class member variables
    Private m_sFileName As String = String.Empty
    Private m_sTitle As String = String.Empty
    Private m_sLink As String = String.Empty
    Private m_sDescription As String = String.Empty
    'constants
    Private Const mksXmlHeader As String = "<?xml version=""1.0"" ?>"
    Private Const mksRssHeader As String = "<rss version=""2.0"">"
    Private Const mksChannelClose As String = "</channel>"
    Private Const mksRssClose As String = "</rss>"

#End Region

#Region " Class Constructors "

    Public Sub New(ByVal RssFileName As String, ByVal FeedTitle As String, ByVal FeedLink As String, ByVal FeedDescription As String)
        'caller is responsible for directory existence and permissions in this sample
        m_sFileName = RssFileName.Trim
        m_sTitle = FeedTitle.Trim
        m_sLink = FeedLink.Trim
        m_sDescription = FeedDescription.Trim
        msRSSHeader = mksXmlHeader & ControlChars.CrLf & mksRssHeader
        msRSSChannel = MakeChannel()
        msRSSFooter = ControlChars.CrLf & ControlChars.Tab & mksChannelClose & ControlChars.CrLf & mksRssClose
    End Sub

#End Region

#Region " Class Public Properties "

    Public ReadOnly Property Description() As String
        Get
            Return m_sDescription
        End Get
    End Property

    Public ReadOnly Property FileName() As String
        Get
            Return m_sFileName
        End Get
    End Property

    Public ReadOnly Property ItemCount() As Integer
        Get
            Return mhtRSSItems.Count
        End Get
    End Property

    Public ReadOnly Property Link() As String
        Get
            Return m_sLink
        End Get
    End Property

    Public ReadOnly Property Title() As String
        Get
            Return m_sTitle
        End Get
    End Property

#End Region

#Region " Class Public Methods "

    'creates an RSS item with Title and Link
    Public Function AddItem(ByVal Title As String, ByVal Link As String) As RssItem
        Dim oItem As New RssItem(Title, Link)
        mhtRSSItems.Add(mhtRSSItems.Count + 1, oItem)
        Return oItem
    End Function

    'creates an RSS item with Description only
    Public Function AddItem(ByVal Description As String) As RssItem
        Dim oItem As New RssItem(Description)
        mhtRSSItems.Add(mhtRSSItems.Count + 1, oItem)
        Return oItem
    End Function

    'write the RSS file to disk, overwriting any existing file
    Public Function SaveRss() As Boolean
        If m_sFileName.Length > 0 And ItemCount > 0 Then
            If Directory.Exists(Path.GetDirectoryName(m_sFileName)) Then
                If WriteFile(msRSSHeader & msRSSChannel & OutputItems() & msRSSFooter) Then
                    Debug.WriteLine("Filename: " & m_sFileName)
                    Debug.Write(msRSSHeader)
                    Debug.Write(msRSSChannel)
                    Debug.Write(OutputItems)
                    Debug.WriteLine(msRSSFooter)
                    Return True
                End If
            End If
        End If
        Return False
    End Function

#End Region

#Region " Class Private Methods "

    'create the channel header section of the RSS file - channel closing tag is in RSSFooter
    Private Function MakeChannel() As String
        Dim sb As New StringBuilder(1024)
        sb.Append(WriteElement("channel", 1))
        sb.Append(WriteElement("title", Title, 2))
        sb.Append(WriteElement("link", Link, 2))
        sb.Append(WriteElement("description", Description, 2))
        Return sb.ToString
    End Function

    'writes RSS Items from newest to oldest
    Private Function OutputItems() As String
        Dim ItemCount As Integer = mhtRSSItems.Count
        Dim item As Integer
        Dim sb As New StringBuilder(1024)
        If ItemCount > 0 Then
            'output newest (highest index) to oldest (lowest index)
            For item = ItemCount To 1 Step -1
                sb.Append(CType(mhtRSSItems.Item(item), RssItem).WriteXML(2))
            Next
        End If
        Return sb.ToString
    End Function

    'writes the string contents to the RSS file
    Private Function WriteFile(ByVal Contents As String) As Boolean
        Try
            Dim sw As StreamWriter
            sw = New StreamWriter(m_sFileName)
            sw.Write(Contents)
            sw.Close()
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

#End Region

End Class

'contains data for an RSS item, with a method to return the item as XML
Public Class RssItem

#Region " Class Declarations "

    Public Title As String = String.Empty
    Public Description As String = String.Empty
    Public Link As String = String.Empty
    Public Author As String = String.Empty
    Public PubDate As String = String.Empty     'should be validated for proper DateTime format
    Public Guid As String = String.Empty        'unique value - usually a link (or permalink)

#End Region

#Region " Class Constructors "

    'require a Title and Link or Description for new items
    Public Sub New(ByVal TitleInit As String, ByVal LinkInit As String)
        Title = TitleInit.Trim
        Link = LinkInit.Trim
    End Sub

    Public Sub New(ByVal DescriptionInit As String)
        Description = DescriptionInit.Trim
    End Sub

#End Region

#Region " Class Public Methods "

    'returns the XML for a complete RSS Item
    Public Function WriteXML(ByVal Indent As Integer) As String
        'Indent is the number of tabs to indent the output
        Dim sb As New StringBuilder(1024)
        sb.Append(WriteElement("item", Indent))
        If Title.Trim.Length > 0 Then
            sb.Append(WriteElement("title", Title.Trim, Indent + 1))
        End If
        If Description.Trim.Length > 0 Then
            sb.Append(WriteElement("description", Description.Trim, Indent + 1))
        End If
        If Link.Trim.Length > 0 Then
            sb.Append(WriteElement("link", Link.Trim, Indent + 1))
        End If
        If Author.Trim.Length > 0 Then
            sb.Append(WriteElement("author", Author.Trim, Indent + 1))
        End If
        If PubDate.Trim.Length > 0 Then
            sb.Append(WriteElement("pubdate", PubDate.Trim, Indent + 1))
        End If
        If Guid.Trim.Length > 0 Then
            sb.Append(WriteElement("guid", Guid.Trim, Indent + 1))
        End If
        sb.Append(WriteElement("/item", Indent))
        Return sb.ToString
    End Function

#End Region

End Class
